/*
 * mt9v022 sensor
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef __MT9V022_H__
#define __MT9V022_H__

struct mt9v022_platform_data {
	unsigned short y_skip_top;	/* Lines to skip at the top */
};

#endif
