#ifndef _DVB_USB_GL861_H_
#define _DVB_USB_GL861_H_

#include "dvb_usb.h"

#define GL861_WRITE		0x40
#define GL861_READ		0xc0

#define GL861_REQ_I2C_WRITE	0x01
#define GL861_REQ_I2C_READ	0x02

#endif
