/*
 * linux/drivers/media/platform/s5p-mfc/s5p_mfc_cmd_v6.h
 *
 * Copyright (C) 2011 Samsung Electronics Co., Ltd.
 *		http://www.samsung.com/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#ifndef S5P_MFC_CMD_V6_H_
#define S5P_MFC_CMD_V6_H_

#include "s5p_mfc_common.h"

struct s5p_mfc_hw_cmds *s5p_mfc_init_hw_cmds_v6(void);

#endif /* S5P_MFC_CMD_H_ */
